/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.antragraum;

import static java.util.Objects.*;

import java.util.stream.Stream;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.NachrichtenManagerProperties;
import de.ozgcloud.nachrichten.postfach.PersistPostfachNachrichtService;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@ConditionalOnProperty(AntragraumProperties.PROPERTY_ANTRAGSRAUM_URL)
public class AntragraumService {

	static final String BAYERN_ID = "BayernID";

	private final PersistPostfachNachrichtService postfachNachrichtService;
	private final Saml2Verifier verifier;
	private final Saml2Parser parser;
	private final Saml2Decrypter decrypter;

	static final String USER_NOTIFICATION_TEMPLATE = """
			Guten Tag,

			bei der Bearbeitung Ihres Antrags hat sich eine Rückfrage ergeben. Bitte beantworten Sie diese im Antragsraum:

			%s

			Vielen Dank,
			Ihre digitale Verwaltung
			""";

	private final AntragraumProperties properties;
	private final NachrichtenManagerProperties nachrichtenManagerProperties;

	@PostConstruct
	void init() {
		if (isNull(nachrichtenManagerProperties.getUrl())) {
			throw new IllegalStateException("Address of Nachrichten-Manager is not set");
		}
	}

	public String getAntragsraumUrl() {
		return properties.getUrl();
	}

	public String getUserNotificationText() {
		return USER_NOTIFICATION_TEMPLATE.formatted(getAntragsraumUrl());
	}

	public Stream<PostfachNachricht> findRueckfragen(String samlToken) {
		verifyToken(samlToken);
		var postfachId = decrypter.decryptPostfachId(parser.parse(samlToken));

		return postfachNachrichtService.findRueckfragen(BAYERN_ID, postfachId);
	}

	public String sendRueckfrageAnswer(String samlToken, String rueckfrageId, PostfachNachricht nachricht) {
		verifyToken(samlToken);

		return postfachNachrichtService.persistAnswer(rueckfrageId, nachricht);
	}

	void verifyToken(String token) {
		var errors = verifier.verify(token);
		if (CollectionUtils.isNotEmpty(errors)) {
			throw new SecurityException("SAML Token verification failed. Errors: %s".formatted(errors));
		}
	}

}
