/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingConstants;
import org.mapstruct.ReportingPolicy;
import org.mapstruct.ValueMapping;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht.Direction;
import de.ozgcloud.vorgang.common.GrpcObject;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN)
public abstract class PostfachNachrichtMapper {

	@Autowired
	private GrpcObjectMapper grpcObjectMapper;

	@Mapping(target = "attachments", source = "attachmentList")
	@Mapping(target = "createdAt", ignore = true)
	@Mapping(target = "createdBy", ignore = true)
	@Mapping(target = "direction", ignore = true)
	@Mapping(target = "messageId", ignore = true)
	@Mapping(target = "sentAt", ignore = true)
	@Mapping(target = "sentSuccessful", ignore = true)
	@ValueMapping(source = "UNRECOGNIZED", target = MappingConstants.NULL)
	@ValueMapping(source = "UNDEFINED", target = MappingConstants.NULL)
	public abstract PostfachNachricht fromGrpcMail(GrpcPostfachMail grpcMail);

	public PostfachAddressIdentifier toPostfachAddressIdentifier(GrpcObject grpcObject) {
		var postfachId = grpcObject.getPropertyList().get(0).getValue(0);
		return StringBasedIdentifier.builder().postfachId(postfachId).build();
	}

	@SuppressWarnings("unchecked")
	public GrpcPostfachMail fromMap(Map<String, Object> mailMap) {
		var postfachMailBuilder = GrpcPostfachMail.newBuilder()
				.setId(MapUtils.getString(mailMap, PostfachNachricht.FIELD_ID))
				.setVorgangId(MapUtils.getString(mailMap, PostfachNachricht.FIELD_VORGANG_ID))
				.setPostfachId(MapUtils.getString(mailMap, PostfachNachricht.FIELD_POSTFACH_ID, StringUtils.EMPTY))
				.setCreatedAt(MapUtils.getString(mailMap, PostfachNachricht.FIELD_CREATED_AT))
				.setCreatedBy(MapUtils.getString(mailMap, PostfachNachricht.FIELD_CREATED_BY, StringUtils.EMPTY))
				.setSentAt(MapUtils.getString(mailMap, PostfachNachricht.FIELD_SENT_AT, StringUtils.EMPTY))
				.setSentSuccessful(MapUtils.getBooleanValue(mailMap, PostfachNachricht.FIELD_SENT_SUCCESSFUL, false))
				.setMessageCode(MapUtils.getString(mailMap, PostfachNachricht.FIELD_MESSAGE_CODE, StringUtils.EMPTY))
				.setDirection(GrpcDirection.valueOf(MapUtils.getString(mailMap, PostfachNachricht.FIELD_DIRECTION)))
				.setSubject(MapUtils.getString(mailMap, PostfachNachricht.FIELD_SUBJECT))
				.setMailBody(MapUtils.getString(mailMap, PostfachNachricht.FIELD_MAIL_BODY))
				.setReplyOption(MapUtils.getString(mailMap, PostfachNachricht.FIELD_REPLY_OPTION))
				.addAllAttachment((Iterable<String>) mailMap.getOrDefault(PostfachNachricht.FIELD_ATTACHMENTS, Collections.emptyList()));

		Optional.ofNullable(getAsMap(mailMap, PostfachNachricht.POSTFACH_ADDRESS_FIELD))
				.filter(MapUtils::isNotEmpty)
				.map(this::buildGrpcPostfachAddress)
				.ifPresent(postfachMailBuilder::setPostfachAddress);

		return postfachMailBuilder.build();
	}

	GrpcPostfachAddress buildGrpcPostfachAddress(Map<String, Object> postfachAddressMap) {
		var postfachAddressBuilder = GrpcPostfachAddress.newBuilder()
				.setType(MapUtils.getIntValue(postfachAddressMap, PostfachAddress.TYPE_FIELD))
				.setVersion(MapUtils.getString(postfachAddressMap, PostfachAddress.VERSION_FIELD))
				.setIdentifier(grpcObjectMapper.fromMap(postfachAddressMap));
		Optional.ofNullable(MapUtils.getString(postfachAddressMap, PostfachAddress.SERVICEKONTO_TYPE_FIELD))
				.ifPresent(postfachAddressBuilder::setServiceKontoType);
		return postfachAddressBuilder.build();
	}

	public PostfachNachricht fromAttachedItem(GrpcVorgangAttachedItem grpcVorgangAttachedItem) {
		return fromMapToPostfachMail(grpcObjectMapper.mapFromGrpc(grpcVorgangAttachedItem.getItem()));
	}

	PostfachNachricht fromMapToPostfachMail(Map<String, Object> mailMap) {
		var postfachMailBuilder = PostfachNachricht.builder()
				.id(MapUtils.getString(mailMap, PostfachNachricht.FIELD_ID))
				.vorgangId(MapUtils.getString(mailMap, (PostfachNachricht.FIELD_VORGANG_ID)))
				.postfachId(MapUtils.getString(mailMap, PostfachNachricht.FIELD_POSTFACH_ID, StringUtils.EMPTY))
				.messageId(MapUtils.getString(mailMap, PostfachNachricht.FIELD_MESSAGE_ID))
				.createdAt(ZonedDateTime.parse(MapUtils.getString(mailMap, PostfachNachricht.FIELD_CREATED_AT)))
				.createdBy(MapUtils.getString(mailMap, PostfachNachricht.FIELD_CREATED_BY))
				.sentAt(getString(mailMap, PostfachNachricht.FIELD_SENT_AT).map(ZonedDateTime::parse).orElse(null))
				.sentSuccessful(MapUtils.getBoolean(mailMap, PostfachNachricht.FIELD_SENT_SUCCESSFUL))
				.messageCode(MapUtils.getString(mailMap, PostfachNachricht.FIELD_MESSAGE_CODE))
				.direction(Direction.valueOf(MapUtils.getString(mailMap, PostfachNachricht.FIELD_DIRECTION)))
				.subject(MapUtils.getString(mailMap, PostfachNachricht.FIELD_SUBJECT))
				.mailBody(MapUtils.getString(mailMap, PostfachNachricht.FIELD_MAIL_BODY))
				.replyOption(PostfachNachricht.ReplyOption.valueOf(MapUtils.getString(mailMap, PostfachNachricht.FIELD_REPLY_OPTION)))
				.attachments(getAsList(mailMap, PostfachNachricht.FIELD_ATTACHMENTS));

		Optional.ofNullable(getAsMap(mailMap, PostfachNachricht.POSTFACH_ADDRESS_FIELD))
				.filter(MapUtils::isNotEmpty)
				.map(this::buildPostfachAddress)
				.ifPresent(postfachMailBuilder::postfachAddress);

		return postfachMailBuilder.build();
	}

	private Optional<String> getString(Map<String, Object> mailMap, String key) {
		return Optional.ofNullable(MapUtils.getString(mailMap, key));
	}

	@SuppressWarnings("unchecked")
	private List<String> getAsList(Map<String, Object> mailMap, String fieldName) {
		return (List<String>) mailMap.getOrDefault(fieldName, Collections.emptyList());
	}

	PostfachAddress buildPostfachAddress(Map<String, Object> postfachAddressMap) {
		return PostfachAddress.builder()
				.type(MapUtils.getIntValue(postfachAddressMap, PostfachAddress.TYPE_FIELD))
				.version(MapUtils.getString(postfachAddressMap, PostfachAddress.VERSION_FIELD))
				.identifier(buildIdentifier(postfachAddressMap))
				.serviceKontoType(MapUtils.getString(postfachAddressMap, PostfachAddress.SERVICEKONTO_TYPE_FIELD))
				.build();
	}

	PostfachAddressIdentifier buildIdentifier(Map<String, Object> postfachAddressMap) {
		return StringBasedIdentifier.builder().postfachId(getPostfachId(postfachAddressMap)).build();
	}

	private String getPostfachId(Map<String, Object> postfachAddressMap) {
		var identifier = getAsMap(postfachAddressMap, PostfachAddress.IDENTIFIER_FIELD);
		return MapUtils.getString(identifier, PostfachNachricht.FIELD_POSTFACH_ID);

	}

	@SuppressWarnings("unchecked")
	Map<String, Object> getAsMap(Map<String, Object> map, String fieldName) {
		return MapUtils.getMap(map, fieldName, Collections.emptyMap());
	}
}